/*
 *
 * This program bitmap_convert.c is intended to illustrate strange 
 * behaviors observed in freetype 2.1.0 and freetype 2.3.7 of the function 
 * FT_Bitmap_Convert()
 * 
 * Usage: bitmap_convert <name of font file> <character to be convert> <render mode> <alignment>
 * 
 * For example: bitmap_convert FreeMono.ttf M FT_RENDER_MODE_LCD 4
 * or bitmap_convert FreeMono.ttf M FT_RENDER_MODE_LCD_V 4
 * Where
 * 1) The render mode is FT_RENDER_MODE_LCD or FT_RENDER_MODE_LCD_V
 *  2) The font files should be available in the directory ./fonts 
 *  
 */

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_BITMAP_H
#include FT_GLYPH_H
#include FT_IMAGE_H
#include <string.h>

#define TRUE 1

int main(int    argc, char** argv)
{
    FT_Library library = NULL;
    FT_Face    face    = NULL;
    FT_Error   error     = 0;
    FT_Error   err_con = 1;
    FT_Glyph   glyph  = NULL;
    FT_BitmapGlyph bglyph = NULL;
    FT_Bitmap *bitmap  = NULL;    
    FT_Bitmap *target     = NULL;
    int char_size[] = {100*64, 0, 100, 0};

    if( argc != 5 )
    {
        printf("Invalid argument.\n");
        printf("USAGE: bitmap_convert <fully pathed font file> <character>  <render_mode> <alignment>\n");
        printf("For example: bitmap_convert fonts/FreeMono.ttf M FT_RENDER_MODE_LCD 4\n");
 	    printf("           or\n");       
	    printf("             bitmap_convert fonts/FreeMono.ttf M FT_RENDER_MODE_LCD_V 4\n");
	    return 0;
    }
    
    char* filename     = argv[1];
    char  char_test    = atoi(argv[2]);
    char *render = argv[3];
    int      render_mode;
    int   alignment    = atoi(argv[4]);
    if(strcmp(render, "FT_RENDER_MODE_LCD") ==0)
    {
        render_mode = 3;
    }
    else if(strcmp(render, "FT_RENDER_MODE_LCD_V") ==0)
    {
        render_mode = 4;
    }
    else
    {
        printf("The value of render mode should be FT_RENDER_MODE_LCD or FT_RENDER_MODE_LCD_V.\n");
        return 0;
    }
	
    error = FT_Init_FreeType( &library );
    
    if( (error != 0) || (library == NULL) )
    {
        printf("Cannot initialize a new FreeType library object.\n");
        return 0;
    }

    error = FT_New_Face( library, filename, 0, &face);
    if( (error != 0) || (face == NULL) )
    {
        printf("Cannot create a handle to face object.\n");
        return 0;
    }

    if( FT_IS_SCALABLE(face) )
    {
        error = FT_Set_Char_Size( face, char_size[0], char_size[1], char_size[2], char_size[3] );
        if( (error != 0) )
        {
            printf("Cannot set the character dimensions of a given face object.");
            return 0;
        }
        FT_Set_Transform( face, 0, 0 );
    }
    
    FT_Load_Char(face, char_test, 0);
    error = FT_Get_Glyph( face->glyph, &glyph );
    if (error != 0)
    {
        printf("Cannot get glyph.\n");
        return 0;
    }
    
    if( (face->glyph->format != FT_GLYPH_FORMAT_BITMAP))
    {
        error = FT_Glyph_To_Bitmap( &glyph,  render_mode, NULL, TRUE);
        if(error != 0)
        {
            printf("Cannot change glyph to bitmap glyph.\n");
            return 0;
        }
        bglyph = (FT_BitmapGlyph)glyph;
        bitmap = &bglyph->bitmap;
    }
    else
    {
        bitmap = &face->glyph->bitmap;
    }
    
    error = FT_Get_Glyph( face->glyph, &glyph );
    if (error != 0)
    {
        printf("Cannot get Glyph.\n");
        return 0;
    }
    
    target = (FT_Bitmap*)malloc( sizeof(FT_Bitmap) );
    if(target == NULL)
    {
        printf("Cannot allocate target bitmap.\n");
        return 0;
    }
    
    FT_Bitmap_New(target);
    
    err_con = FT_Bitmap_Convert(library, bitmap, target, alignment);
    if (err_con !=0)
    {
		printf("The expected return error code is 0, but function returns %d = FT_Err_Invalid_Argument.\n", err_con);
    	printf("The bitmap conversion with render mode = %s was not performed.\n", 
				((render_mode == 3)? "FT_RENDER_MODE_LCD" : "FT_RENDER_MODE_LCD_V"));

    }

    if(target)
    {
         free(target);
    }

    if(face)
    {
        FT_Done_Face(face);
    }  
    
    if(library)
    {
        FT_Done_FreeType(library);
    }
    	
    return 0;
}
